from flask import Flask, request, json
from flask_cors import CORS
from .logger import init_logger
from app.config import Config
import logging
import os
from flask_sqlalchemy import SQLAlchemy
from flask_bcrypt import Bcrypt
from flask_marshmallow import Marshmallow
from flask_socketio import SocketIO

db = SQLAlchemy()
bcrypt = Bcrypt()
ma = Marshmallow()
socketio = SocketIO(cors_allowed_origins="*", engineio_logger=True, ping_timeout=30, ping_interval=15)

if os.environ.get("ENVIRONMENT") == "LOCAL":
    log = init_logger("App", log_level=logging.DEBUG)
else:
    log = init_logger("App", log_level=logging.INFO)

environments = {"LOCAL": "local", "STAGING": "staging", "PROD": "production"}

def dict_json_response(data, status):
    if type(data) is dict:
        if request.headers.getlist("X-Forwarded-For"):
            ip_string = request.headers.getlist("X-Forwarded-For")[0]
            data["ip"] = ip_string.split(",")[0]
        else:
            data["ip"] = request.remote_addr
    resp = json.dumps(data)
    return resp, status, {"Content-Type": "application/json; charset=utf-8"}

class App(Flask):
    def __init__(self, *args, **kwargs):
        super(App, self).__init__(*args, **kwargs)

def boot_app(config_class=Config):
    app = App(__name__)

    app.config.from_object(config_class)

    db.init_app(app)
    bcrypt.init_app(app)

    socketio.init_app(app)
    CORS(app, resources={r"/*": {"origins": "*"}})

    @app.before_request
    def before_req():
        print("before request")

    from app.zone.routes import zones
    from app.sensor_data.routes import sensor_data, map_data, settings
    from app.users.routes import users
    from app.devices.routes import devices
    from app.rooms.routes import rooms
    from app.customer.routes import customer
    from app.Site.routes import site
    from app.roles.routes import roles
    from app.device_types.routes import device_types
    from app.gateways.routes import gateways
    from app.logs.routes import logs
    from app.patients.routes import patient
    from app.location_log.routes import locationlogs


    app.register_blueprint(zones)
    app.register_blueprint(sensor_data)
    app.register_blueprint(map_data)
    app.register_blueprint(settings)
    app.register_blueprint(users)
    app.register_blueprint(devices)
    app.register_blueprint(rooms)
    app.register_blueprint(customer)
    app.register_blueprint(site)
    app.register_blueprint(roles)
    app.register_blueprint(device_types)
    app.register_blueprint(gateways)
    app.register_blueprint(logs)
    app.register_blueprint(patient)
    app.register_blueprint(locationlogs)

    return app, socketio